#include "OgreDepthShadowMapRenderingRun.h"
#include "OgreIlluminationManager.h"


OgreDepthShadowMapRenderingRun::OgreDepthShadowMapRenderingRun(OgreSharedRuns* sharedRuns,
																String name,
																Light* light,
																unsigned int resolutionX,
																unsigned int resolutionY,
																String materialName
														   )
														   :DepthShadowMapRenderingRun(resolutionX, resolutionY)
														   , OgreRenderingRun(1, 1)
														   , RenderingRun(1, 1)
{
	this->light = light;
	this->sharedRuns = sharedRuns;
	this->name = name;	
	
	this->materialName = materialName;

	createDepthMap();	
}

void OgreDepthShadowMapRenderingRun::createDepthMap()
{
	
		TexturePtr texPtr = Ogre::TextureManager::getSingleton().createManual(name, 
																			"default",
																			TEX_TYPE_2D,
																			resolutionX,
																			resolutionY,
																			0,
																			0,
																			PF_FLOAT16_RGBA,
																			TU_RENDERTARGET);
		depthMapTexture = texPtr.getPointer();
		depthMapCamera = Root::getSingleton()._getCurrentSceneManager()->createCamera(name + "_CAMERA");
		//add viewport to rendertarget
		HardwarePixelBuffer* hpb = (depthMapTexture->getBuffer()).getPointer();
		RenderTarget* rt = hpb->getRenderTarget();
		Viewport* v = rt->addViewport(depthMapCamera);
		v->setOverlaysEnabled(false);
		rt->setAutoUpdated(false);	
}

void OgreDepthShadowMapRenderingRun::updateFrame(unsigned long frameNum)
{
	refreshLight(frameNum);

	if(light->getType() == Light::LT_POINT)
	{
		updateDepthMap();
	}
	else
	{
		updateDepthMap();
	}
	
	
}

void OgreDepthShadowMapRenderingRun::updateDepthCubeFace(int facenum)
{
}

void OgreDepthShadowMapRenderingRun::updateDepthMap()
{
	setMaterialForVisibles(materialName, depthMapCamera, false);	
	RenderTarget* rt = depthMapTexture->getBuffer().getPointer()->getRenderTarget();	
	Viewport* vp = rt->getViewport(0);
	
	rt->update();
	restoreMaterials();
	//rt->writeContentsToFile("shadowmap1.dds");
}

void setnear()
{

}

void OgreDepthShadowMapRenderingRun::refreshLight(unsigned long frameNum)
{
	
	if(light!= 0)
	{
		if(light->getType() == Light::LT_DIRECTIONAL)
		{
			Camera* viewcam = OgreIlluminationManager::getSingleton().getMainCamera();
			
			OgreIlluminationManager::getSingleton().updatePerLightRun(light->getName(),
																		ILLUMRUN_FOCUSING_MAP,
																		frameNum);

			OgreFocusingMapRenderingRun* frun = (OgreFocusingMapRenderingRun*)
				OgreIlluminationManager::getSingleton().getPerLightRun(light->getName(), ILLUMRUN_FOCUSING_MAP)->asOgreRenderingRun();
			
			Vector3 min;
			Vector3 max;
			frun->getMinMax(min, max);

			Vector3 lightpos = light->getPosition();
			Vector3 lightdir = light->getDirection();
			lightdir.normalise();

			depthMapCamera->setProjectionType(PT_ORTHOGRAPHIC);	
			/*depthMapCameras[0]->setAspectRatio(1.0);
			depthMapCameras[0]->setFOVy(Radian(Degree(20)));
			depthMapCameras[0]->setNearClipDistance(0.1);
			depthMapCameras[0]->setFarClipDistance(200);*/

			
			Vector3 scale = (max - min);
			scale *= 1.5;
			//scale = Vector3(150,150,150);
			Matrix4 projection = depthMapCamera->getProjectionMatrix();
			projection = Matrix4::IDENTITY;
			projection.setScale(Vector3(2.0 / scale.x, 2.0 / scale.y, -2.0 / scale.z));			
			projection.setTrans(Vector3(-(max.x + min.x) / scale.x,
										-(max.y + min.y) / scale.y,
										-(max.z + min.z) / scale.z));
			depthMapCamera->setCustomProjectionMatrix(true, projection);
			
			depthMapCamera->setPosition(lightpos);
			depthMapCamera->setDirection(lightdir);
			
	/*
			projection = Matrix4::IDENTITY;
			projection.setScale(Vector3(1.0 / 100.0, 1.0 / 100.0, -1.0 / 100.0));
			depthMapCameras[0]->setCustomProjectionMatrix(true, projection);
			depthMapCameras[0]->setPosition( Vector3(50,50,50) );*/

		}
		else if(light->getType() == Light::LT_SPOTLIGHT)
		{
	/*		Vector3 dir = light->getDirection();
			depthMapCamera->setDirection( dir );
			Vector3 pos = light->getParentNode()->getWorldPosition();
			depthMapCamera->setPosition(pos);
			depthMapCamera->setProjectionType(PT_PERSPECTIVE);		
			depthMapCamera->setFOVy(light->getSpotlightOuterAngle());*/
		}
		else//point light
		{
	/*		Vector3 pos = light->getParentNode()->getWorldPosition();
			Vector3 dir = -pos;
			depthMapCamera->setDirection( dir );
			depthMapCamera->setPosition(pos);
			depthMapCamera->setProjectionType(PT_PERSPECTIVE);
			depthMapCamera->setFOVy(Radian(Degree(140)));
			depthMapCamera->setNearClipDistance(1);
			// depthMapCamera->setFarClipDistance(200);
	
			//OGRE_EXCEPT(0, "NOT implemented for Pointlight", "OgreDepthShadowMapRenderingRun::refreshLight");
		*/	
			/*Vector3 pos = light->getParentNode()->getWorldPosition();
			Vector3 dir = -pos;
			depthMapCamera->setDirection( dir );
			depthMapCamera->setPosition(pos);
			depthMapCamera->setProjectionType(PT_ORTHOGRAPHIC);
			depthMapCamera->*/
			
		}
	}
}

Matrix4 OgreDepthShadowMapRenderingRun::getLightViewMatrix()
{
	return depthMapCamera->getViewMatrix();
}

Matrix4 OgreDepthShadowMapRenderingRun::getLightViewProjMatrix()
{
	return depthMapCamera->getProjectionMatrixWithRSDepth() * depthMapCamera->getViewMatrix();
}

